package org.openliberty.openaz.pep;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.security.Permission;

import org.openliberty.openaz.azapi.constants.AzCategoryId;
import org.openliberty.openaz.azapi.constants.AzCategoryIdResource;
import org.openliberty.openaz.azapi.constants.AzCategoryIdAction;
import org.openliberty.openaz.azapi.constants.AzXacmlStrings;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Default Java Permission implementation of <code>JavObjectMapper</code>.
 * It supports the mapping of 
 * <ul>
 * <li>
 * a Java Permission object
 * </ul>
 * to 
 * <ul>
 * <li>
 * <code>a AzWrapperRequestObject</code> 
 * </ul>
 * containing 
 * <ul>
 * <li>
 * a XACML AttributeId and an AttributeValue using the Permission classname as a XACML az-resource-type, 
 * <li>
 * and using the Permission.getName() as XACML resource-id,
 * <li>
 * and the Permission.getActions() as the XACML action-id
 * </ul>
 * Basically, a typical Java Permission is converted to three XACML name,value
 * pairs of AttributeId and AttributeValue.
 * @author Josh Bregman, Rich Levinson, Prateek Mishra
 */
public class SimpleJavaPermissionMapper implements JavaObjectMapper {
	private static Set<Class> supportedClasses =
		new HashSet<Class>() {{ add(Permission.class); }};
	Log log = LogFactory.getLog(this.getClass()); 
    public SimpleJavaPermissionMapper() {
        super();
    }
    /**
     * Returns the Set of Classes supported by this Mapper.
     * @return a set of classes
     */
    public Set<Class> getSupportedClasses(){
    	return supportedClasses;
    }

    /**
     * Determines if the object requested to map is a Java 
     * Permission.
     * 
     * @param obj An Object
     * @return a boolean set to true if obj is a Java Permission, ow false
     */
    public boolean canMapObject(Object obj) {
    	log.info("canMap(Object obj): " + obj.getClass().getName());
        if (obj instanceof Permission){
        	log.info("obj instanceof Permission == true");
            return true;
        } else {
        	log.info("Cannot map obj instanceof: " + obj.getClass().getName());
            return false;
        }
    }

    /**
     * Maps a Java Permission object to a set of XACML attributes.
     * The Java Permission maps as follows:
     * <pre>
     *   - The value of Permission.getName() maps to the value of
     *      the Resource Attribute with AttributeId:
     *        "urn:oasis:names:tc:xacml:1.0:resource:resource-id"
     *   - The value of Permission.getClass().getName() maps to 
     *      the value of a Resource Attribute with AttributeId:
     *       "org.openliberty.openaz.pep.resource.resource-type"
     *   - The value of Permission.getActions() maps to the value of
     *      the Action Attribute with AttributeId:
     *       "urn:oasis:names:tc:xacml:1.0:action:implied-action"  
     * </pre>
     */
    public <T extends Enum<T> & AzCategoryId>
    	AzWrapperRequestObject<T> map(
    				Object javaObject,
            		AzWrapperRequestObject<T> azWrapperRequestObject) 
    			throws PepException {
		log.info("Object to map: " + 
		javaObject.getClass().getName());
		if (javaObject instanceof Permission) {
			return convertPermission(
					(Permission)javaObject,
					azWrapperRequestObject);
		}		
		throw new PepException(
			"Can't map an object of class "+
			javaObject.getClass().getName());
}
    
    private <T extends Enum<T> & AzCategoryId> 
    	AzWrapperRequestObject<T > convertPermission(
    				Permission javaPermission,
    				AzWrapperRequestObject<T> azWrapperRequestObject)
    		throws PepException {
    	String attributeId = null;
    	if ( azWrapperRequestObject.wrappedObject.getAzCategoryId().
    			equals(AzCategoryIdResource.AZ_CATEGORY_ID_RESOURCE)) {
    		attributeId = AzXacmlStrings.X_ATTR_RESOURCE_ID;
    		azWrapperRequestObject.setAttribute(
    				attributeId, 
    				javaPermission.getName());
    		azWrapperRequestObject.setAttribute(
    				"org.openliberty.openaz.pep.resource.resource-type",
    				javaPermission.getClass().getName());
    	}
    	else if ( azWrapperRequestObject.wrappedObject.getAzCategoryId().
    			equals(AzCategoryIdAction.AZ_CATEGORY_ID_ACTION)) {
    		attributeId = AzXacmlStrings.X_ATTR_ACTION_ID;
    		azWrapperRequestObject.setAttribute(
    				attributeId,
    				javaPermission.getActions());
    	}
    	else {
    		throw new PepException(
    			"Can't convert Java Permission to wrapped object " +
    			" that is not Resource or Action; This wrapped object " +
    			" has AzCategoryId: " +
    			azWrapperRequestObject.wrappedObject.getAzCategoryId() +
    			"and is of type: " + 
    			azWrapperRequestObject.wrappedObject.getClass().getName());
    	}
    	return azWrapperRequestObject;
    }

}
